/*  toba.h -- general definitions  */

/* Copyright 1996 Arizona Board of Regents; see COPYRIGHT file for details. */


#include <setjmp.h>	/* used by generated code for try/catch */


/* Java types are capitalized. Sometimes they match C types. */
typedef void			Void;
typedef unsigned char		Boolean;
typedef unsigned short		Char;
typedef signed char		Byte;
typedef signed short		Short;
typedef signed int		Int;
typedef unsigned int		UInt;	/* only for >>>; not Java primitive */

#if (0x40000000lu << 2) == 0		/* if long is just 32 bits wide */
    typedef signed long long	Long;
    typedef unsigned long long	ULong;	/* only for >>>; not Java primitive */
#else
    typedef signed long		Long;
    typedef unsigned long	ULong;	/* only for >>>; not Java primitive */
#endif

typedef float			Float;
typedef double			Double;
typedef void *			Object;
typedef struct class *    	Class;
typedef struct monitor *        Monitor;


/* generic object instance struct */
struct in_generic { Class class; Monitor monitor; };

/* class struct header */
/* note: any changes here also require mods to code generator and helpers.c */
struct class {			
    int needinit;		/* does class need to be initialized? */
    int flags;			/* isInterface, isArray, isPrimitive, etc. */
    Object name;		/* class name (a String) */
    struct in_generic classclass; /* instance of class Class */
    int instsize;		/* size of an instance or element */
    int nmethods;		/* number of methods (in method table) */
    int nsupers;		/* number of superclasses incl self */
    const Class *supers;	/* list of superclasses incl self */
    int ninters;		/* number of interfaces implemented */
    int ndinters;		/* number of directly implemented interfaces */
    const Class *inters;	/* list of interfaces */
    int nothers;		/* number of other known classes */
    const Class *others;	/* list of other known classes */
    Class arrayclass;		/* array of this class, when needed */
    Class elemclass;		/* class of contained elements, if array */
    Monitor monitor; 		/* monitor for static synchronized methods */
    void (*initializer)();	/* class initializer (<clinit>), if any */
    void (*constructor)();	/* no-arg instance initializer */
    void (*finalizer)();	/* instance finalizer function */
};

/* generic method and class structs */
struct mt_generic { int s; Void(*f)(); };
struct cl_generic { struct class C; struct mt_generic M[1]; };

/* array instance structs */
struct carray
    { Class class; Monitor monitor; int length; int pad; Char   data[1]; };
struct barray 
    { Class class; Monitor monitor; int length; int pad; Byte   data[1]; };
struct sarray 
    { Class class; Monitor monitor; int length; int pad; Short  data[1]; };
struct iarray 
    { Class class; Monitor monitor; int length; int pad; Int    data[1]; };
struct larray 
    { Class class; Monitor monitor; int length; int pad; Long   data[1]; };
struct farray 
    { Class class; Monitor monitor; int length; int pad; Float  data[1]; };
struct darray 
    { Class class; Monitor monitor; int length; int pad; Double data[1]; };
struct aarray 
    { Class class; Monitor monitor; int length; int pad; Object data[1]; };

/* character array macro */
#define CARRAY(len) struct { void *c; Monitor m; int n; int pad; Char d[len]; }

/* primitive "classes" */
extern struct class cl_boolean;
extern struct class cl_char;
extern struct class cl_byte;
extern struct class cl_short;
extern struct class cl_int;
extern struct class cl_long;
extern struct class cl_float;
extern struct class cl_double;

/* array-of-primitive classes */
extern struct cl_java_lang_Object acl_boolean;
extern struct cl_java_lang_Object acl_char;
extern struct cl_java_lang_Object acl_byte;
extern struct cl_java_lang_Object acl_short;
extern struct cl_java_lang_Object acl_int;
extern struct cl_java_lang_Object acl_long;
extern struct cl_java_lang_Object acl_float;
extern struct cl_java_lang_Object acl_double;

/* exception handling table */
struct handler {
    Class class;		/* exception */
    unsigned short start_pc;
    unsigned short end_pc;
    unsigned short handler_pc;
};



/* thread-specific info */
struct mythread {
    void *jmpbuf;		/* current setjmp buffer */
    Object exception;		/* current exception */
    jmp_buf default_jmpbuf;	/* initial jump buffer */
    int exception_count;	/* number of uncaught exceptions */
};



/* structs for floating point constants */
union fconst { UInt b;  Float v;  };	/* float */
union dconst { ULong b; Double v; };	/* double */



/* macros for control flow changes */
#define GOTO(oldpc,newlbl) goto newlbl
#define GOBACK(oldpc,newlbl) goto newlbl
#define RETTO(oldpc,lblnum) do { tgt = lblnum; goto TOP; } while (0)



/* helper functions called by generated code */

/* start main program */
int start(Class c, Void main(Object o), int argc, char *argv[]);

/*
 *  handle class initialization: synchronize, check circularity,
 *     init superclass, call myclass.<clinit>, catch exceptions, etc.
 */
void initclass(Class myclass);

/* get current thread's per-thread structure */
struct mythread *mythread();

/* search exception table to find handler */
int findhandler(struct handler *table, int tsize, Object exception, int oldpc);

/* find interface method for "invokeinterface" */
void (*findinterface(Object o, int signaturehash))();

/* synchronization */
void monitorenter(Object o, struct mythread *thr, int val, volatile int *resp);
void monitorexit(Object o, struct mythread *thr, int val, volatile int *resp);
void enterclass(Class c, struct mythread *thr, int val, volatile int *resp);
void exitclass(Class c, struct mythread *thr, int val, volatile int *resp);

/* throw exception */
void athrow(Object o);
void throwArrayIndexOutOfBoundsException(Object o, int index);
void throwClassCastException(Object o);
void throwDivisionByZeroException(void);
void throwInstantiationException(Object o);
void throwNoSuchMethodError(Object o);
void throwNullPointerException(char *mesg);

/* create object, also creating class structs for arrays as needed */
Object new(Class c);
Object anewarray(Class c, int nelem);
Object mnewarray(Class c, int arraydim, int ndim, /*d1,d2,*/...);

/* check class relationships */
int instanceof(Object o, Class c, int ndim);

/* floating point math operations */
Int dtoi(Double d);
Long dtol(Double d);
Double remdr(Double x, Double y);
