/*  helpers.c -- native runtime helpers called from generated code  */

/* Copyright 1996 Arizona Board of Regents; see COPYRIGHT file for details. */

#include "toba.h"
#include "runtime.h"
#include "java_lang_NullPointerException.h"

#include <math.h>


/*  findhandler(table, size, exceptn, oldpc) -- search table to find handler  */

int findhandler(struct handler *h, int tsize, Object exception, int oldpc)
{
    if (exception == NULL)	/* if NULL, mutate to NullPointerException */
	exception = construct(&cl_java_lang_NullPointerException.C);

    while (tsize-- > 0) {
	if ((unsigned)oldpc >= h->start_pc
	&&  (unsigned)oldpc <  h->end_pc
	&& (h->class == 0 || instanceof(exception, h->class, 0)))
	    return h->handler_pc;
	h++;
    }
    return -1;
}



/*  findinterface(o, n) -- find interface method for "invokeinterface"  */

void (*findinterface(Object o, int signaturehash))()
{
    int i;
    Class c = ((struct in_generic *)o)->class;
    struct mt_generic *m = ((struct cl_generic *)c)->M;

    for (i = c->nmethods - 1; i >= 0; i--) {
	if (m->s == signaturehash)
	    return m->f;
	m++;
    }

    throwIncompatibleClassChangeError(o);
    /*NOTREACHED*/
}



/* 
 *  instanceof(o, c, n) -- is o an instance of an array of n dims of class c?
 *
 *  o is any object, but not null.
 *  c is any simple class or pseudo-class, but not an array class.
 *
 *  o is checked for assignability to an n-dimensional array of c.
 *  If n is zero, o is checked for assignability to an object of type c.
 */ 
int instanceof(Object o, Class c, int ndim)
{
    int i;
    Class oc = ((struct in_generic *)o)->class;

    while (ndim > 0) {
	oc = oc->elemclass;		/* find underlying element classes */
	if (!oc)
	    return 0;			/* dimension mismatch */
	ndim--;
    }

    if (oc == c)			/* quick check for exact match */
	return 1;

    if ((oc->flags | c->flags) & IS_PRIMITIVE)
	return 0;			/* if primitive, must be identical */

    if (c->flags & IS_INTERFACE) {
	for (i = 0; i < oc->ninters; i++)
	    if (oc->inters[i] == c)
		return 1;
	return 0;
    }

    /* at this point c is a simple class although oc may be an array; */
    /* an array matches only java.lang.Object */
    return oc->nsupers >= c->nsupers && oc->supers[oc->nsupers-c->nsupers] == c;
}



/*
 *  dtoi(d) -- convert double (or float) to int.
 */
Int dtoi(Double d)
{
    if (d != d)		/* if NaN */
    	return 0;
    else if (d >= MAX_JAVA_INT)
        return MAX_JAVA_INT;
    else if (d <= MIN_JAVA_INT)
        return MIN_JAVA_INT;
    else
    	return (Int)d;
}

/*
 *  dtol(d) -- convert double (or float) to long.
 */
Long dtol(Double d)
{
    if (d != d)		/* if NaN */
    	return 0;
    else if (d >= MAX_JAVA_LONG)
        return MAX_JAVA_LONG;
    else if (d <= MIN_JAVA_LONG)
        return MIN_JAVA_LONG;
    else
    	return (Long)d;
}

/*
 *  remdr(a,b) -- compute a % b.
 */
Double remdr(Double a, Double b)
{
    if (a != a || b != b || b == 0)
    	return 0.0 / 0.0;	/* NaN */
    else
    	return fmod(a, b);
}
