/* Copyright 1996 Arizona Board of Regents; see COPYRIGHT file for details. */

#include <stddef.h>
#include "toba.h"
#include "runtime.h"
#include "java_lang_Class.h"
#include "java_lang_Long.h"
#include "java_util_Hashtable.h"

/* hash table with (classname -> instance of class Class) */
static struct in_java_util_Hashtable *all_classes = 0;

/* return the Class associated with a java.lang.Class instance */
static Class
find_class(struct in_java_lang_Class *clcl)
{
    Class ret;

    if(!clcl)
        throwNullPointerException("find class");

    /* 
     * the classclass instance is embedded in a Class - just subtract
     * the offset to get the corresponding Class
     */
    ret = (Class) ((char *)clcl - offsetof(struct class, classclass));
    return ret;
}

/* return the java.lang.Class instance associated with a Class */
static Object
find_classclass(Class cl)
{
    /*
     * The in_java_lang_Class structure is embedded in a Class
     * it is made up of the pointer to its type descriptor.
     * the address of the pointer is the Object instance
     */
    return &cl->classclass;
}

/* recursively find all classes accessed by this class */
static void
find_all_classes(Class c)
{
    Object val;
    int i;

    /* java_util_Hashtable get(Object, Object) */
    val = get_O_doAa7(all_classes, c->name);
    if(!val) {
        /* we haven't added this one yet */
        /* java_util_Hashtable put(Object, Object) */
        put_OO_4Xy3U(all_classes, c->name, find_classclass(c));
        for(i = 0; i < c->nsupers; i++)
            find_all_classes(c->supers[i]);
        for(i = 0; i < c->ninters; i++)
            find_all_classes(c->inters[i]);
        for(i = 0; i < c->nothers; i++)
            find_all_classes(c->others[i]);
    }
}

/* external entry point to tell us which class started the ball rolling */
void
set_main_class(Class mainclass)
{
    /* make a hashtable w/ 250 buckets to start with */
    all_classes = new(&cl_java_util_Hashtable.C);
    /* java_util_Hashtable Hashtable(int) */
    init_i_yOEvX(all_classes, 250);

    find_all_classes(mainclass);
}

/* java/lang/Class forName (Ljava/lang/String;)Ljava/lang/Class; */
Object forName_S_UuOsX(Object Harg1) 
{
    struct in_java_lang_String *arg1 = (struct in_java_lang_String *)Harg1;
    struct in_java_lang_Class *val;

    /* STATIC Method */
    init_java_lang_Class();

    /* look it up in our hash table of all known classes */
    /* java_util_Hashtable get(Object, Object) */
    val = (struct in_java_lang_Class *)get_O_doAa7(all_classes, arg1);

    if(!val)
        throwClassNotFoundException(cstring(arg1));

    return val;
}

/* java/lang/Class newInstance ()Ljava/lang/Object; */
Object newInstance__GLAhk(Object Harg1) 
{
    struct in_java_lang_Class *this = (struct in_java_lang_Class *)Harg1;
    Class cl;
    Object res;

    cl = find_class(this);
    /* we're never allowed to newInstance java.lang.Class
     * since all instances *must* be embedded within struct Class
     */
    if(cl == &cl_java_lang_Class.C)
        throwIllegalAccessException("can't instantiate java.lang.Class");

    res = construct(cl);
    return res;
}

/* java/lang/Class getName ()Ljava/lang/String; */
Object getName__KWUaL(Object Harg1) 
{
    struct in_java_lang_Class *this = (struct in_java_lang_Class *)Harg1;
    Class cl;

    cl = find_class(this);

    /* Strings are immutable, we can return this one without making a copy */
    return cl->name;
}

/* java/lang/Class getSuperclass ()Ljava/lang/Class; */
Object getSuperclass__bNIlH(Object Harg1) 
{
    struct in_java_lang_Class *this = (struct in_java_lang_Class *)Harg1;
    Class cl, super;

    /* do we have a superclass? */
    cl = find_class(this);
    if(cl->nsupers < 2)
        return 0;

    super = cl->supers[1];
    return find_classclass(super);
}

/* java/lang/Class getInterfaces ()[Ljava/lang/Class; */
Object getInterfaces__156xb(Object Harg1) 
{
    struct in_java_lang_Class *this = (struct in_java_lang_Class *)Harg1;
    struct aarray *inters;
    Class c;
    int i;

    c = find_class(this);
    inters = anewarray(&cl_java_lang_Class.C, c->ndinters);

    for(i = 0; i < c->ndinters; i++) {
        inters->data[i] = find_classclass(c->inters[i]);
    }
    return inters;
}

/* java/lang/Class getClassLoader ()Ljava/lang/ClassLoader; */
Object getClassLoader__Gl1Ig(Object Harg1) 
/*ARGSUSED*/
{
    /* 
     * right now class loaders are not supported
     * so no class passed in could possibly have had
     * a class loader
     */
    return 0;
}

/* java/lang/Class isInterface ()Z */
Boolean isInterface__so6R0(Object Harg1) 
{
    struct in_java_lang_Class *this = (struct in_java_lang_Class *)Harg1;
    Class cl;

    cl = find_class(this);
    if(cl->flags & IS_INTERFACE)
        return JAVA_TRUE;
    else
        return JAVA_FALSE;
}

