/* Copyright 1996 Arizona Board of Regents; see COPYRIGHT file for details. */

#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/utsname.h>
#include <unistd.h>
#include <pwd.h>
#include <assert.h>

#include "toba.h"
#include "runtime.h"
#include "java_lang_System.h"
#include "java_lang_Error.h"
#include "java_util_Hashtable.h"
#include "java_util_Properties.h"

/* java/lang/System currentTimeMillis ()J */
Long currentTimeMillis__gBRCJ() 
{
    Long res;
    struct timeval tv;

    /* STATIC Method */
    init_java_lang_System();

    gettimeofday(&tv, 0);
    res = (Long)tv.tv_usec / 1000 + (Long)tv.tv_sec * 1000;
    return res;
}

/* java/lang/System arraycopy (Ljava/lang/Object;ILjava/lang/Object;II)V */
Void arraycopy_OiOii_JRbqy(Object Harg1, Int src_pos, Object Harg3, Int dst_pos, Int len) 
{
    struct barray *src = (struct barray *)Harg1;
    struct barray *dst = (struct barray *)Harg3;
    int elemsize;

    /* STATIC Method */
    init_java_lang_System();

    if(!src || !dst)
        throwNullPointerException("arraycopy");

    /* src and dst must both be arrays */
    if(!(src->class->flags & IS_ARRAY) ||
       !(dst->class->flags & IS_ARRAY))
        throwArrayStoreException("Object is not an array");

    /* pos and length must be in range */
    if(src_pos < 0 || src_pos > src->length ||
       len < 0 || src_pos+len > src->length ||
       dst_pos < 0 || dst_pos > dst->length ||
       dst_pos+len > dst->length)
        throwIndexOutOfBoundsException("bad bounds for arraycopy");
    
    /* make sure its legal to do the copy between arrays */
    if(instanceof(src, dst->class->elemclass, 1)) {
       /* 
        * element types are either equivalent or the
        * destination type is a superclass of the source type.
        * the copy is guaranteed to be legal and we can do a
        * memmove (but not memcpy, because src and dst may overlap).
        */
        elemsize = src->class->instsize;
        memmove(dst->data + dst_pos * elemsize,
               src->data + src_pos * elemsize,
               len * elemsize);
    } else if(instanceof(dst, src->class->elemclass, 1)) {
        /*
         * the element type of the source is a super type of
         * the element type of the destination.  Copies
         * are legal only if each copied element from the src
         * is already of a type assignable to the dest array.
         *
         * We know that we have arrays of objects at this point.
         */
        struct aarray *asrc = (struct aarray *)src;
        struct aarray *adst = (struct aarray *)dst;

        while(len--) {
            /* note: asrc->data[src_pos] can be a null reference */
            if(asrc->data[src_pos] &&
               !instanceof(asrc->data[src_pos], adst->class->elemclass, 0))
                throwArrayStoreException("incompatible element types");
            
            adst->data[dst_pos++] = asrc->data[src_pos++];
        }
    } else {
        /* the types didnt match */
        throwArrayStoreException("incompatible element types");
    }
}

struct prop {
    char *key, *val;
};

/* java/lang/System initProperties (Ljava/util/Properties;)Ljava/util/Properties; */
Object initProperties_P_hGJ5i(Object Harg1) 
{
    static struct prop prop[] = {
        { "java.vendor",        TOBA_VENDOR },
        { "java.vendor.url",    TOBA_URL },
        { "java.vendor.version",TOBA_VERSION },
        { "java.version",       JAVA_VERSION },
        { "java.class.version", CLASS_VERSION },
        { "file.separator",     FILE_SEPARATOR },
        { "path.separator",     PATH_SEPARATOR },
        { "line.separator",     LINE_SEPARATOR },
    /* 
     * These properties are discovered dynamically :
     *   os.name    os.arch     os.version
     *   user.name  user.home   user.dir
     *   java.home  java.class.path
     */
        { 0, 0 } /*last*/
    };
    struct in_java_util_Properties *arg1 = 
        (struct in_java_util_Properties *)Harg1;
    Object (*putfunc)(Object, Object, Object);
    struct utsname uts;
    struct passwd *pw;
    int i, res;
    char *cp, buf[MAXPATHLEN + 1];

    /* STATIC Method */
    init_java_lang_System();

    /* get the method for put() from Properties' method table */
    putfunc = cl_java_util_Properties.M.put_OO_4Xy3U.f;

    /* sanity - putfunc should be the same as java_util_Hashtable.put() */
    if(putfunc != put_OO_4Xy3U) {
        throwMesg(&cl_java_lang_Error.C, 
                   "initProperites: API lib out of date");
    }

    /* static properties */
    for(i = 0; prop[i].key; i++) {
        putfunc(arg1, javastring(prop[i].key), 
                      javastring(prop[i].val));
    }

    /* dynamic properties */
    res = uname(&uts);
    putfunc(arg1, javastring("os.name"),
                  javastring(res != -1 ? uts.sysname : ""));
    putfunc(arg1, javastring("os.version"),
                  javastring(res != -1 ? uts.release : ""));
    putfunc(arg1, javastring("os.arch"),
                  javastring(res != -1 ? uts.machine : ""));

    pw = getpwuid(getuid());
    putfunc(arg1, javastring("user.name"),
                  javastring(pw ? pw->pw_name : "unknown"));
    putfunc(arg1, javastring("user.home"),
                  javastring(pw ? pw->pw_dir : "unknown"));

    cp = getcwd(buf, MAXPATHLEN + 1);
    putfunc(arg1, javastring("user.dir"),
                  javastring(cp ? cp : ""));

    cp = getenv("CLASSPATH");
    putfunc(arg1, javastring("java.class.path"),
                  javastring(cp ? cp : ""));

    cp = getenv("JAVA_HOME");
    putfunc(arg1, javastring("java.home"),
                  javastring(cp ? cp : ""));

    return arg1;
}

