/* runtime.h -- prototypes and definitions for native code helpers */

/* Copyright 1996 Arizona Board of Regents; see COPYRIGHT file for details. */



#include "java_lang_String.h"
#include "java_io_FileDescriptor.h"
#include "java_net_InetAddress.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>



/* implementation identification */
#define JAVA_VERSION 	"1.0.2"		/* Java spec version */
#define CLASS_VERSION	"45.3"		/* Java classfile version */
#define TOBA_VENDOR	"The University of Arizona"
#define TOBA_URL	"http://www.cs.arizona.edu/sumatra/"
#define TOBA_VERSION	"1.0.b4"	/* our own version number */

/* default configuration */
#define DEFAULT_HEAP	(1024 * 1024)	/* default initial heap size */

/* unix separator characters */
#define PATH_SEPARATOR	":"
#define FILE_SEPARATOR	"/"
#define LINE_SEPARATOR	"\n"

/* class flags */
#define IS_INTERFACE 0x0001	/* class is an interface class */
				/* (MUST BE 1; assumed so by Opcode.java) */
#define IS_ARRAY     0x0002	/* class represents an array */
#define IS_PRIMITIVE 0x0004	/* class represents a primitive type */

/* java.lang.Object initializer & finalizer */
#define OBJ_INIT init__AAyKx
#define OBJ_FINI finalize__UKxhs
#define OBJ_METHODS (cl_java_lang_Object.M)

/* Java representations */
#define JAVA_FALSE	0
#define JAVA_TRUE	1
#define MIN_JAVA_INT ((Int) 0x80000000)
#define MAX_JAVA_INT ((Int) 0x7FFFFFFF)
#define MIN_JAVA_LONG ((Long)((ULong)1 << 63))
#define MAX_JAVA_LONG (~MIN_JAVA_LONG)



/* prototypes, excluding those listed in toba.h for use by generated code */

/* alloc.c */
void meminit(void);
void memexit(void);
void *allocate(int nbytes);
void finalizer(void *address, void (*func)(void *));
Object construct(Class c);

/* file.c */
Void fd_open(struct in_java_io_FileDescriptor *fd, Object unameo, int oflags);
Int fd_read_char(struct in_java_io_FileDescriptor *fd);
Int fd_read_bytes
    (struct in_java_io_FileDescriptor *fd, Object Harg2, Int off, Int len);
Void fd_close(struct in_java_io_FileDescriptor *fd);
Void fd_write_char(struct in_java_io_FileDescriptor *fd, Int arg2);
Void fd_write_bytes
    (struct in_java_io_FileDescriptor *fd, Object Harg2, Int off, Int len);
int name_stat(struct in_java_lang_String *uname, struct stat *buf);
Boolean name_access(struct in_java_lang_String *uname, int amode);
int java_fd(int unixfd);
int unix_fd(int javafd);
int java_fd_valid(int javafd);
void set_address
   (struct in_java_net_InetAddress *jaddr, Int port, struct sockaddr_in *uaddr);

/* finalizer.c */
void finalization_init(void);
void finalizer_queue_prealloc();
void needs_finalization(Object o, void (*f)(void*));

/* helpers.c */
/* (all functions are prototyped in toba.h) */

/* monitor.c */
void monitor_package_init(void);
void monitorwait(void *address, Long timeout);
void monitornotify(void *address);
void monitornotifyall(void *address);
void fixup_monitors(void);

/* runtime.c */
void start_thread(Void startfunc(Object o), Object arg);
void terminate(int status);
void fatal(char *mesg);
void unimpl(char *name);
char *cstring(struct in_java_lang_String *str);
struct in_java_lang_String *javastring(char *cstr);

/* structs.c */
void initstructs(void);

/* java_lang_Class.c */
void set_main_class(Class c);

/* throw.c */
void throwArrayIndexOutOfBoundsException(Object o, int index);
void throwArrayStoreException(char *mesg);
void throwClassCastException(Object o);
void throwClassCircularityError(Object o);
void throwClassNotFoundException(char *mesg);
void throwEOFException(char *mesg, int xerrno);
void throwExceptionInInitializerError(Object o);
void throwFileNotFoundException(char *mesg, int xerrno);
void throwIOException(char *mesg, int xerrno);
void throwIllegalAccessException(char *mesg);
void throwIncompatibleClassChangeError(Object o);
void throwIndexOutOfBoundsException(char *mesg);
void throwInstantiationException(Object o);
void throwInterruptedIOException(char *mesg, int xerrno);
void throwMesg(Class etype, char *mesg);
void throwMesgErrno(Class etype, char *mesg, int xerrno);
void throwNegativeArraySizeException(int size);
void throwNoSuchMethodError(Object o);
void throwNumberFormatException(char *mesg);
void throwOutOfMemoryError(int nbytes);
void throwUnknownHostException(char *mesg);
