/*  throw.c -- runtime exception tossers for Toba  */

/* Copyright 1996 Arizona Board of Regents; see COPYRIGHT file for details. */


#include <stdio.h>
#include <string.h>

#include "toba.h"
#include "runtime.h"

#include "java_lang_Throwable.h"
#include "java_lang_Error.h"
#include "java_lang_NullPointerException.h"
#include "java_lang_NegativeArraySizeException.h"
#include "java_lang_IndexOutOfBoundsException.h"
#include "java_lang_ArrayIndexOutOfBoundsException.h"
#include "java_lang_NumberFormatException.h"
#include "java_lang_ArithmeticException.h"
#include "java_lang_ArrayStoreException.h"
#include "java_lang_OutOfMemoryError.h"
#include "java_lang_NoSuchMethodError.h"
#include "java_lang_InstantiationException.h"
#include "java_lang_IncompatibleClassChangeError.h"
#include "java_lang_ClassCastException.h"
#include "java_lang_ClassCircularityError.h"
#include "java_lang_ClassNotFoundException.h"
#include "java_lang_IllegalAccessException.h"
#include "java_io_IOException.h"
#include "java_io_FileNotFoundException.h"
#include "java_io_InterruptedIOException.h"
#include "java_io_EOFException.h"
#include "java_net_UnknownHostException.h"


/*
 *  athrow(o) -- throw an arbitrary exception
 *
 *  called from generated code and from below.
 */
void athrow(Object o)
{
    struct mythread *thr = mythread();

    thr->exception = o;
    longjmp(thr->jmpbuf, 1);
}


/*  throwMesg(c, mesg) -- throw an exception with a descriptive message  */

void throwMesg(Class etype, char *mesg)
{
    struct in_java_lang_Throwable *e;

    e = (struct in_java_lang_Throwable *)construct(etype);
    if(mesg)
        e->detailMessage = (Object)javastring(mesg);
    athrow(e);
}


/*  throwMesgErrno(c, mesg, n) -- throw exception including errno information */

void
throwMesgErrno(Class etype, char *mesg, int xerrno)
{
    char buf[256], *errmsg;

    errmsg = strerror(xerrno);
    if(!errmsg || strlen(mesg) + strlen(errmsg) + 3 > 256u) {
        throwMesg(etype, mesg);
    } else {
        sprintf(buf, "%s: %s", mesg, errmsg);
        throwMesg(etype, buf);
    }
}



/*  throw specific exceptions  */


void throwNullPointerException(char *mesg)
{
    throwMesg(&cl_java_lang_NullPointerException.C, mesg);
}


void throwNegativeArraySizeException(int size)
{
    char buf[25];

    sprintf(buf, "negative array size %d", size);
    throwMesg(&cl_java_lang_NegativeArraySizeException.C, buf);
}


void throwIndexOutOfBoundsException(char *mesg)
{
    throwMesg(&cl_java_lang_IndexOutOfBoundsException.C, mesg);
}


void throwArrayIndexOutOfBoundsException(Object o, int index)
/*ARGSUSED*/
{
    throwMesg(&cl_java_lang_ArrayIndexOutOfBoundsException.C, 0);
}


void throwDivisionByZeroException(void)
{
    throwMesg(&cl_java_lang_ArithmeticException.C, "division by zero");
}


void throwIOException(char *mesg, int xerrno)
{
    throwMesgErrno(&cl_java_io_IOException.C, mesg, xerrno);
}


void throwFileNotFoundException(char *mesg, int xerrno)
{
    throwMesgErrno(&cl_java_io_FileNotFoundException.C, mesg, xerrno);
}


void throwEOFException(char *mesg, int xerrno)
{
    throwMesgErrno(&cl_java_io_EOFException.C, mesg, xerrno);
}


void throwInterruptedIOException(char *mesg, int xerrno)
{
    throwMesgErrno(&cl_java_io_InterruptedIOException.C, mesg, xerrno);
}


void throwNumberFormatException(char *mesg)
{
    throwMesg(&cl_java_lang_NumberFormatException.C, mesg);
}


void throwArrayStoreException(char *mesg)
{
    throwMesg(&cl_java_lang_ArrayStoreException.C, mesg);
}


void throwOutOfMemoryError(int nbytes)
{
    char buf[50];

    sprintf(buf, "Out of memory allocating %d bytes", nbytes);
    throwMesg(&cl_java_lang_OutOfMemoryError.C, buf);
}


void throwUnknownHostException(char *mesg)
{
    throwMesg(&cl_java_net_UnknownHostException.C, mesg);
}


void throwClassCastException(Object o)
/*ARGSUSED*/
{
    throwMesg(&cl_java_lang_ClassCastException.C, 0);
}


void throwNoSuchMethodError(Object o)
/*ARGSUSED*/
{
    throwMesg(&cl_java_lang_NoSuchMethodError.C, 0);
}


void throwInstantiationException(Object o)
/*ARGSUSED*/
{
    throwMesg(&cl_java_lang_InstantiationException.C, 0);
}


void throwClassCircularityError(Object o)
/*ARGSUSED*/
{
    throwMesg(&cl_java_lang_ClassCircularityError.C, 0);
}


void throwClassNotFoundException(char *mesg)
{
    throwMesg(&cl_java_lang_ClassNotFoundException.C, mesg);
}

void throwIllegalAccessException(char *mesg)
{
    throwMesg(&cl_java_lang_IllegalAccessException.C, mesg);
}


void throwIncompatibleClassChangeError(Object o)
/*ARGSUSED*/
{
    throwMesg(&cl_java_lang_IncompatibleClassChangeError.C, 0);
}


/* ExceptionInInitializerError doesn't seem to exist yet; throw Error */

void throwExceptionInInitializerError(Object o)
/*ARGSUSED*/
{
    throwMesg(&cl_java_lang_Error.C, "exception in initializer");
}
