//  ByteCode -- method code structures

//  Copyright 1996 Arizona Board of Regents; see COPYRIGHT file for details.



package toba;

import java.io.*;
import java.util.*;



class ByteCode {

// instance variables
int max_stack;		// maximum stack used
int max_locals;		// maximum locals used
byte code[];		// raw bytecode
short pcmap[];		// pc to instruction mapping
Instr instrs[];		// instruction list
Handler handlers[];	// exception table



// construct a ByteCode instance from a raw code attribute

ByteCode(byte[] attr)
{
    // unpack the Code attribute
    ByteArrayInputStream b = new ByteArrayInputStream(attr);
    DataInputStream d = new DataInputStream(b);

    try {
	max_stack = d.readUnsignedShort();
	max_locals = d.readUnsignedShort();

	code = new byte[d.readInt()];
	d.readFully(code);

	// exception table
	handlers = new Handler[d.readUnsignedShort()];
	for (int i = 0; i < handlers.length; i++) {
	    Handler c = new Handler();
	    c.start = d.readUnsignedShort();
	    c.end = d.readUnsignedShort();
	    c.jump = d.readUnsignedShort();
	    c.type = d.readUnsignedShort();
	    handlers[i] = c;
	}
    } catch (IOException e) {
	Misc.abort("bad code attribute");
    }

    // construct instruction list
    Instr ins;
    Vector insvec = new Vector();
    pcmap = new short[code.length];

    for (int pc = 0; pc < code.length; pc += ins.length) {
	pcmap[pc] = (short)insvec.size();
	ins = new Instr(code, pc);
	insvec.addElement(ins);
    }
    instrs = new Instr[insvec.size()];
    insvec.copyInto(instrs);
}



}
