//  Debug.java -- debugging support
//
//  Debugging flags are set by the "-dxxx" command option.
//  Debugging output is written on standard output.

//  Copyright 1996 Arizona Board of Regents; see COPYRIGHT file for details.



package toba;

import java.io.*;

class Debug {



//  command line debugging options
static final char All = 'a';	
static final char General = 'g';

static final char Constants = 'k';	// dump constant table of initial class
static final char Trace = 't';		// trace major actions by translator
static final char Temp = 'x';		// for temporary debugging hacks
static final char Code = 'c';		// dump analyzed bytecode as a unit
static final char Instrs = 'i';		// list instrs with generated code

static boolean flag[] = new boolean[256];



//  select(s) -- set debugging options based on flag characters

static void select(String s)
{
    int i;

    // set selected flags
    for (i = 0; i < s.length(); i++)
	flag[(int)s.charAt(i)] = true;

    // any options at all imply general debugging
    flag[(int)General] = true;

    // option 'a' implies all flags
    if (flag[(int)All])
	for (i = 0; i < flag.length; i++)
	    flag[i] = true;
}



//  selected(c) -- check if a particular debugging char is selected

static boolean selected(char c)
{
    return flag[(int)c];
}



} // class Debug
