//  Files.java -- general file operations

//  Copyright 1996 Arizona Board of Regents; see COPYRIGHT file for details.



package toba;

import java.io.*;



class Files {

static final int cBufferSize = 8192;
static final int hBufferSize = 8192;
static final int classBufferSize = 8192;
static final int peekBufferSize = 1024;


private static String fileheader = "/*  created by " + Misc.PROGNAME + "  */";



//  findclass(name) -- find .class file for a class.

static DataInputStream findclass(String name) throws ClassNotFoundException
{
    // check for illegal name that might be mapped into legal file name later
    if (name.indexOf(File.separator) >= 0)
	throw new ClassNotFoundException(name);

    // get CLASSPATH; construct file name
    String classpath = System.getProperty("java.class.path")+File.pathSeparator;
    String filepath = name.replace('.', File.separatorChar) + ".class";

    // search CLASSPATH for .class file
    int i = 0;
    int j;
    while ((j = classpath.indexOf(File.pathSeparator, i)) >= 0) {
	String dir = classpath.substring(i, j);
	try {
	    return instream(dir + File.separator + filepath, classBufferSize);
	} catch (FileNotFoundException e) {
	};
	i = j + 1;
    }

    // as a last resort, strip off package name and check ./name.class
    i = name.lastIndexOf(".");
    if (i >= 0) {
	try {
	    return instream(name.substring(i + 1) + ".class", classBufferSize);
	} catch (FileNotFoundException e) {
	};
    }

    // no luck
    throw new ClassNotFoundException(name);
}



//  instream(filename, bufsize) -- create buffered DataInputStream.

static DataInputStream instream(String filename, int bufsize)
    throws FileNotFoundException
{
    return new DataInputStream(
	new BufferedInputStream(new FileInputStream(filename), bufsize));
}



//  pkgpath(classname) -- extract path to directory holding class file.

private static String pkgpath(String classname)
{
    int i = classname.lastIndexOf(".");
    if (i < 0)
	return "";
    else
	return classname.substring(0, i).replace('.', File.separatorChar);
}



//  classtail(classname) -- extract tail component of class name.

private static String classtail(String classname)
{
    int i = classname.lastIndexOf(".");
    if (i < 0)
	return classname;
    else
	return classname.substring(i + 1);
}



//  oopen(classname, suffix, bufsize) -- open output file for generated code.

static PrintStream oopen(String classname, String suffix, int bufsize)
{
    String filename = Names.classfile(classname) + suffix;
    File f = new File(filename);

    if (foreign(f))
	Misc.abort(filename + ": contents unrecognized; will not overwrite");

    if (f.exists() && !f.canWrite())
	Misc.abort(filename + ": cannot write");

    try {
	PrintStream d = new PrintStream(
	    new BufferedOutputStream(new FileOutputStream(f), bufsize));
	d.println("/*  " + f.getName() + " -- from Java class " 
	    + classname + "  */");
	d.println(fileheader);
	return d;
    } catch(Exception e) {
	Misc.abort(filename + ": cannot open for output");
    }
    return null;
}



//  foreign(f) -- is file f a "foreign" file (not to be overwritten)?

private static boolean foreign(File f)
{
    if (!f.exists() || f.length() == 0)
	return false;
    try {
	DataInputStream d = instream(f.getName(), peekBufferSize);
	d.readLine();
	String s = d.readLine();
	d.close();
	return !s.equals(fileheader);
    } catch(Exception e) {
	return false;
    }
}



} // class Files
