//  Main.java -- overall control of Toba

//  Copyright 1996 Arizona Board of Regents; see COPYRIGHT file for details.



package toba;

import java.io.*;

class Main {



static ClassInfo current;	// global pointer to current class
static Constant[] ctab;		// global pointer to its constant table

static boolean echo_names;	// echo file names when compiling?



//  Main(args) -- main program.

public static void main(String args[]) 
{
    int i;

    options(args);

    Opcode.init();

    for (i = 0; i < args.length; i++)
	if (args[i] != null)
	    doname(args[i]);
}


//  options(args[]) -- record options and null them in arglist.

//  very simple -- really need a "getopt" to do this right

private static void options(String args[])
{
    int i;
    String s;
    char c;
    int nopts = 0;

    for (i = 0; i < args.length; i++) {
	s = args[i];
	if (s.length() < 2 || s.charAt(0) != '-')
	    continue;
	nopts++;
	args[i] = null;
	c = s.charAt(1);
	switch (c) {
	    case 'd':
		Debug.select(s.substring(2));
		break;
	    default:
		Misc.abort("usage: toba [-dxxxx] classname...");
	}
    }

    if (args.length - nopts > 1)
	echo_names = true;
}


//  doname(name) -- process one class, given by file or class name.

private static void doname(String name)
{
    ClassInfo c = null;

    if (echo_names)
	System.err.println(name + ":");

    try {
	c = ClassInfo.load(name);
    } catch (ClassNotFoundException e) {
	Misc.abort("cannot load class " + name);
    } catch (FileNotFoundException e) {
	Misc.abort("cannot load file " + e.getMessage());
    } catch (IOException e) {
	Misc.abort("I/O error loading " + name + ": " + e);
    }

    current = c;
    ctab = c.constants;

    PrintStream cstream = Files.oopen(c.name, ".c", Files.cBufferSize);
    PrintStream hstream = Files.oopen(c.name, ".h", Files.hBufferSize);

    if (Debug.selected(Debug.Constants))
	Constant.dumptable(cstream, c.constants);

    HFile.write(hstream, c);
    hstream.close();

    CFile.write(cstream, c);
    cstream.close();

    ctab = null;
    c.retain();
}



} // class Main
