//  Misc.java -- miscellaneous methods

//  Copyright 1996 Arizona Board of Regents; see COPYRIGHT file for details.



package toba;

import java.io.*;

class Misc {



// manifest constants

static final String PROGNAME = "Toba";

static final int CHAR_INDX_SIZE = 128;	// size of array indexed by (C) chars

// code generation parameters

static final String JVM_TYPES="ailfd";	//  Java Virtual Machine datatypes

static final int DECLS_PER_LINE = 15;	// variables per declaration line
static final int GCHARS_PER_LINE = 18;	// generated char consts on one line
static final int GCHARS_PER_INIT = 10;	// generated char consts on initial line




//  abort(s) -- abort run, with a message.

static void abort(String s)
{
    System.err.print("   ");
    System.err.println(s);
    System.exit(1);
}



//  isQuotable(c) -- is character c reasonably specified as 'c'?

static boolean isQuotable(char c)
{
    return c >= ' ' && c <= '~' && c != '\'' && c != '\\';
}



//  rettype(m) -- Compute C return type of Java method m.

static String rettype(Field m)
{
    String s = m.signature;
    return ctype(s.charAt(s.indexOf(')') + 1));
}



//  ctype(c) -- return C type indicated by character c.
//
//  upper case characters are Java signature characters.
//  lower case characters are JVM datatypes.

static String ctype(char c)
{
    switch (c) {
	case 'B':             return "Byte";
	case 'C':             return "Char";
	case 'D':  case 'd':  return "Double";
	case 'F':  case 'f':  return "Float";
	case 'I':  case 'i':  return "Int";
	case 'J':  case 'l':  return "Long";
	case 'S':             return "Short";
	case 'V':             return "Void";
	case 'Z':             return "Boolean";
	default:              return "Object";
    }
}



} // class Misc
